import 'package:comunic/helpers/conversations_helper.dart';
import 'package:comunic/helpers/groups_helper.dart';
import 'package:comunic/helpers/users_helper.dart';
import 'package:comunic/helpers/webapp_helper.dart';
import 'package:comunic/lists/groups_list.dart';
import 'package:comunic/lists/memberships_list.dart';
import 'package:comunic/lists/users_list.dart';
import 'package:comunic/models/membership.dart';
import 'package:comunic/ui/widgets/account_image_widget.dart';
import 'package:comunic/ui/widgets/group_icon_widget.dart';
import 'package:comunic/ui/widgets/safe_state.dart';
import 'package:comunic/utils/date_utils.dart';
import 'package:comunic/utils/intl_utils.dart';
import 'package:comunic/utils/ui_utils.dart';
import 'package:flutter/material.dart';

/// Memberships panel
///
/// @author Pierre Hubert

class MembershipsPanel extends StatefulWidget {
  @override
  _MembershipsPanelState createState() => _MembershipsPanelState();
}

const _MembershipIconsWidth = 30.0;

class _MembershipsPanelState extends SafeState<MembershipsPanel> {
  final _refreshKey = GlobalKey<RefreshIndicatorState>();
  MembershipList _membershipList;
  UsersList _usersList;
  GroupsList _groupsList;

  Future<void> _refresh() async {
    try {
      final memberships = await WebAppHelper.getMemberships();
      final users = await UsersHelper().getListWithThrow(memberships.usersId);
      final groups = await GroupsHelper().getListOrThrow(memberships.groupsId);

      setState(() {
        _membershipList = memberships;
        _usersList = users;
        _groupsList = groups;
      });
    } catch (e, s) {
      print("Could not load the list of memberships! $e\n$s");
      setTimeout(5, _refresh);
    }
  }

  @override
  void initState() {
    super.initState();
    _refresh();
  }

  @override
  Widget build(BuildContext context) {
    if (_membershipList == null) return buildCenteredProgressBar();

    return RefreshIndicator(
        key: _refreshKey, child: _buildMembershipsList(), onRefresh: _refresh);
  }

  Widget _buildMembershipsList() {
    return ListTileTheme(
      iconColor: IconTheme.of(context).color,
      child: ListView.builder(
        itemBuilder: _buildMembershipTile,
        itemCount: _membershipList.length,
      ),
    );
  }

  Widget _buildMembershipTile(BuildContext context, int index) {
    final membership = _membershipList[index];

    switch (membership.type) {
      case MembershipType.FRIEND:
        return _buildFriendMembership(membership);

      case MembershipType.GROUP:
        return _buildGroupMembership(membership);

      case MembershipType.CONVERSATION:
        return _buildConversationMembership(membership);
    }
  }

  // TODO : add support for requested friendship
  // TODO : add private messages icon support
  Widget _buildFriendMembership(Membership membership) {
    final user = _usersList.getUser(membership.friend.id);
    final connected = membership.friend.isConnected;
    return ListTile(
      leading: AccountImageWidget(user: user, width: _MembershipIconsWidth),
      title: Text(user.displayName),
      subtitle: Text(
        connected ? tr("Online") : diffTimeFromNowToStr(membership.lastActive),
        style: TextStyle(color: connected ? Colors.green : null),
      ),
    );
  }

  // TODO : add support for pending groups memberships
  Widget _buildGroupMembership(Membership membership) {
    final group = _groupsList.getGroup(membership.groupID);
    return ListTile(
      leading: GroupIcon(
        group: group,
        width: _MembershipIconsWidth,
      ),
      title: Text(group.displayName),
      subtitle: Text(diffTimeFromNowToStr(membership.lastActive)),
    );
  }

  Widget _buildConversationMembership(Membership membership) {
    return ListTile(
      dense: true,
      leading: Icon(Icons.message),
      title: Text(ConversationsHelper.getConversationName(
          membership.conversation, _usersList)),
      subtitle: Text(diffTimeFromNowToStr(membership.lastActive)),
    );
  }
}
